/**
* DevExtreme (ui/file_manager.d.ts)
* Version: 19.2.7 (build 20092-0323)
* Build date: Wed Apr 01 2020
*
* Copyright (c) 2012 - 2020 Developer Express Inc. ALL RIGHTS RESERVED
* Read about DevExtreme licensing here: https://js.devexpress.com/Licensing/
*/
import '../jquery_augmentation';

import {
    dxElement
} from '../core/element';

import {
    dxContextMenuItem
} from './context_menu';

import {
    dxDataGridColumn
} from './data_grid';

import {
    dxToolbarItem
} from './toolbar';

import Widget, {
    WidgetOptions
} from './widget/ui.widget';

/** Warning! This type is used for internal purposes. Do not import it directly. */
export interface dxFileManagerOptions extends WidgetOptions<dxFileManager> {
   /** Specifies the allowed upload file extensions. */
    allowedFileExtensions?: Array<string>;
   /** Configures the context menu settings. */
    contextMenu?: dxFileManagerContextMenu;
   /** Specifies the path that is used when the FileManager is initialized. */
    currentPath?: string;
   /** Customizes columns in details view. Applies only if itemView.mode is "details". */
    customizeDetailColumns?: ((columns: Array<dxDataGridColumn>) => Array<dxDataGridColumn>);
   /** Allows you to provide custom icons to be used as thumbnails. */
    customizeThumbnail?: ((fileItem: any) => string);
   /** Specifies the file provider. */
    fileProvider?: any;
   /** Configures the file and folder view. */
    itemView?: { mode?: 'details' | 'thumbnails', showFolders?: boolean, showParentFolder?: boolean };
   /** A function that is executed when the current directory is changed. */
    onCurrentDirectoryChanged?: ((e: { component?: dxFileManager, element?: dxElement, model?: any }) => any);
   /** A function that is executed when the selected file is opened. */
    onSelectedFileOpened?: ((e: { component?: dxFileManager, element?: dxElement, model?: any, fileItem?: any }) => any);
   /** Specifies actions that a user is allowed to perform on files and folders. */
    permissions?: { copy?: boolean, create?: boolean, download?: boolean, move?: boolean, remove?: boolean, rename?: boolean, upload?: boolean };
   /** Specifies the root folder name. */
    rootFolderName?: string;
   /** Specifies whether a user can select a single or multiple files and folders in the item view simultaneously. */
    selectionMode?: 'multiple' | 'single';
   /** Configures toolbar settings. */
    toolbar?: dxFileManagerToolbar;
   /** Configures upload settings. */
    upload?: { maxFileSize?: number };
}/** [tags] ctp The FileManager is a widget that allows users to upload, select, and manage files and directories in different file storages. #include common-ctp-note with { component: "FileManager" } */
export default class dxFileManager extends Widget {
    constructor(element: Element, options?: dxFileManagerOptions)
    constructor(element: JQuery, options?: dxFileManagerOptions)
   /** Gets the current directory object. */
    getCurrentDirectory(): any;
   /** Gets the selected items. */
    getSelectedItems(): Array<any>;
   /** Reloads data and repaints the widget. */
    refresh(): Promise<any> & JQueryPromise<any>;
}

/** Warning! This type is used for internal purposes. Do not import it directly. */
export interface dxFileManagerContextMenu {
   /** Configures context menu items' settings. */
    items?: Array<dxFileManagerContextMenuItem | 'create' | 'upload' | 'refresh' | 'download' | 'move' | 'copy' | 'rename' | 'delete'>;
}

/** Warning! This type is used for internal purposes. Do not import it directly. */
export interface dxFileManagerContextMenuItem extends dxContextMenuItem {
   /** Specifies the context menu item's name. */
    name?: 'create' | 'upload' | 'refresh' | 'download' | 'move' | 'copy' | 'rename' | 'delete' | string;
   /** Specifies the context menu item's visibility. */
    visible?: boolean;
}

/** Warning! This type is used for internal purposes. Do not import it directly. */
export interface dxFileManagerToolbar {
   /** Configures settings of the toolbar items that are visible when users select files. */
    fileSelectionItems?: Array<dxFileManagerToolbarItem | 'showNavPane' | 'create' | 'upload' | 'refresh' | 'viewSwitcher' | 'download' | 'move' | 'copy' | 'rename' | 'delete' | 'clear' | 'separator'>;
   /** Configures toolbar items' settings. */
    items?: Array<dxFileManagerToolbarItem | 'showNavPane' | 'create' | 'upload' | 'refresh' | 'viewSwitcher' | 'download' | 'move' | 'copy' | 'rename' | 'delete' | 'clear' | 'separator'>;
}

/** Warning! This type is used for internal purposes. Do not import it directly. */
export interface dxFileManagerToolbarItem extends dxToolbarItem {
   /** Specifies the toolbar item's location. */
    location?: 'after' | 'before' | 'center';
   /** Specifies the toolbar item's name. */
    name?: 'showNavPane' | 'create' | 'upload' | 'refresh' | 'viewSwitcher' | 'download' | 'move' | 'copy' | 'rename' | 'delete' | 'clear' | 'separator' | string;
   /** Specifies the toolbar item's visibility. */
    visible?: boolean;
}

declare global {
interface JQuery {
    dxFileManager(): JQuery;
    dxFileManager(options: "instance"): dxFileManager;
    dxFileManager(options: string): any;
    dxFileManager(options: string, ...params: any[]): any;
    dxFileManager(options: dxFileManagerOptions): JQuery;
}
}
/** Warning! This type is used for internal purposes. Do not import it directly. */
export type Options = dxFileManagerOptions;

/** @deprecated use Options instead */
/** Warning! This type is used for internal purposes. Do not import it directly. */
export type IOptions = dxFileManagerOptions;