/**
* DevExtreme (viz/chart_components/base_chart.d.ts)
* Version: 19.2.7 (build 20092-0323)
* Build date: Wed Apr 01 2020
*
* Copyright (c) 2012 - 2020 Developer Express Inc. ALL RIGHTS RESERVED
* Read about DevExtreme licensing here: https://js.devexpress.com/Licensing/
*/
import '../../jquery_augmentation';

import {
    dxElement
} from '../../core/element';

import {
    template
} from '../../core/templates/template';

import DataSource, {
    DataSourceOptions
} from '../../data/data_source';

import {
    event
} from '../../events';

import {
    format
} from '../../ui/widget/ui.widget';

import {
    basePointObject,
    baseSeriesObject,
    chartSeriesObject,
    dxChartAnnotationConfig,
    dxChartSeriesTypesCommonSeriesLabel,
    dxChartSeriesTypesCommonSeriesPoint
} from '../chart';

import {
    BaseLegend,
    BaseLegendItem
} from '../common';

import BaseWidget, {
    BaseWidgetOptions,
    BaseWidgetTooltip
} from '../core/base_widget';

/** Warning! This type is used for internal purposes. Do not import it directly. */
export interface BaseChartOptions<T = BaseChart> extends BaseWidgetOptions<T> {
   /** Specifies adaptive layout options. */
    adaptiveLayout?: BaseChartAdaptiveLayout;
   /** Specifies animation options. */
    animation?: { duration?: number, easing?: 'easeOutCubic' | 'linear', enabled?: boolean, maxPointCountSupported?: number } | boolean;
   /** Customizes the appearance of an individual point label. */
    customizeLabel?: ((pointInfo: any) => dxChartSeriesTypesCommonSeriesLabel);
   /** Customizes the appearance of an individual series point. */
    customizePoint?: ((pointInfo: any) => dxChartSeriesTypesCommonSeriesPoint);
   /** Binds the widget to data. */
    dataSource?: Array<any> | DataSource | DataSourceOptions | string;
   /** Specifies options of the legend. */
    legend?: BaseChartLegend;
   /** A function that is executed when all series are ready. */
    onDone?: ((e: { component?: T, element?: dxElement, model?: any }) => any);
   /** A function that is executed when a series point is clicked or tapped. */
    onPointClick?: ((e: { component?: T, element?: dxElement, model?: any, jQueryEvent?: JQueryEventObject, event?: event, target?: basePointObject }) => any) | string;
   /** A function that is executed after the pointer enters or leaves a series point. */
    onPointHoverChanged?: ((e: { component?: any, element?: any, target?: basePointObject }) => any);
   /** A function that is executed when a series point is selected or selection is canceled. */
    onPointSelectionChanged?: ((e: { component?: any, element?: any, target?: basePointObject }) => any);
   /** A function that is executed when a tooltip becomes hidden. */
    onTooltipHidden?: ((e: { component?: T, element?: dxElement, model?: any, target?: basePointObject | dxChartAnnotationConfig | any }) => any);
   /** A function that is executed when a tooltip appears. */
    onTooltipShown?: ((e: { component?: T, element?: dxElement, model?: any, target?: basePointObject | dxChartAnnotationConfig | any }) => any);
   /** Sets the palette to be used for colorizing series and their elements. */
    palette?: Array<string> | 'Bright' | 'Default' | 'Harmony Light' | 'Ocean' | 'Pastel' | 'Soft' | 'Soft Pastel' | 'Vintage' | 'Violet' | 'Carmine' | 'Dark Moon' | 'Dark Violet' | 'Green Mist' | 'Soft Blue' | 'Material' | 'Office';
   /** Specifies what to do with colors in the palette when their number is less than the number of series (in the Chart widget) or points in a series (in the PieChart widget). */
    paletteExtensionMode?: 'alternate' | 'blend' | 'extrapolate';
   /** Specifies whether a single point or multiple points can be selected in the chart. */
    pointSelectionMode?: 'multiple' | 'single';
   /** Specifies options for series. */
    series?: any | Array<any>;
   /** Configures tooltips. */
    tooltip?: BaseChartTooltip;
}
/** Warning! This type is used for internal purposes. Do not import it directly. */
export interface BaseChartAdaptiveLayout {
   /** Specifies the minimum container height at which the layout begins to adapt. */
    height?: number;
   /** Specifies whether point labels should be kept when the widget adapts the layout. */
    keepLabels?: boolean;
   /** Specifies the minimum container width at which the layout begins to adapt. */
    width?: number;
}
/** Warning! This type is used for internal purposes. Do not import it directly. */
export interface BaseChartLegend extends BaseLegend {
   /** Allows you to change the order, text, and visibility of legend items. */
    customizeItems?: ((items: Array<BaseChartLegendItem>) => Array<BaseChartLegendItem>);
   /** Specifies an SVG element that serves as a custom legend item marker. */
    markerTemplate?: template | ((legendItem: BaseChartLegendItem, element: SVGGElement) => string | SVGElement | JQuery);
}
/** Warning! This type is used for internal purposes. Do not import it directly. */
export interface BaseChartTooltip extends BaseWidgetTooltip {
   /** Formats the point argument before it is displayed in the tooltip. To format the point value, use the format option. */
    argumentFormat?: format;
   /** Specifies a custom template for a tooltip. */
    contentTemplate?: template | ((pointInfo: any, element: dxElement) => string | Element | JQuery);
   /** Allows you to change tooltip appearance. */
    customizeTooltip?: ((pointInfo: any) => any);
   /** Specifies whether the tooltip is shared across all series points with the same argument. */
    shared?: boolean;
}
/** Warning! This type is used for internal purposes. Do not import it directly. *//** A base class for all chart widgets included in the ChartJS library. */
export class BaseChart extends BaseWidget {
    constructor(element: Element, options?: BaseChartOptions)
    constructor(element: JQuery, options?: BaseChartOptions)
   /** Deselects the chart's selected series. The series is displayed in an initial style. */
    clearSelection(): void;
   /** Gets all the series. */
    getAllSeries(): Array<baseSeriesObject>;
    getDataSource(): DataSource;
   /** Gets a series with a specific name. */
    getSeriesByName(seriesName: any): chartSeriesObject;
   /** Gets a series with a specific index. */
    getSeriesByPos(seriesIndex: number): chartSeriesObject;
   /** Hides all widget tooltips. */
    hideTooltip(): void;
   /** Reloads data and repaints the widget. */
    refresh(): void;
    render(): void;
   /** Redraws the widget. */
    render(renderOptions: any): void;
}

/** Warning! This type is used for internal purposes. Do not import it directly. */
export interface BaseChartLegendItem extends BaseLegendItem {
   /** The series that the item represents on the legend. */
    series?: baseSeriesObject;
}
